from typing import Optional
from pydantic import BaseModel, Field


class IscountsResponse(BaseModel):
    id_: Optional[int] = Field(default=None, alias="id")
    objectName: Optional[str] = None
    create: Optional[str] = None
    update: Optional[str] = None
    sevClient: Optional[str] = None
    discount: Optional[str] = None
    text: Optional[str] = None
    percentage: Optional[str] = None
    value: Optional[str] = None
    isNet: Optional[str] = None
    class Config:
        populate_by_name = True


from typing import Optional, Any
from pydantic import BaseModel, Field


class ReceiptGuideDto(BaseModel):
    accountDatevId: Optional[int] = None
    accountNumber: Optional[str] = None
    accountName: Optional[str] = None
    description: Optional[str] = None
    allowedTaxRules: Optional[Any] = None
    allowedReceiptTypes: Optional[Any] = None
    class Config:
        populate_by_name = True


from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.sevclient import SevClient

class DocumentSettings(BaseModel):
    """Document Settings für PDF-Generierung"""
    id_: Optional[str] = Field(default=None, alias="id")
    objectName: Optional[str] = None
    additionalInformation: Optional[str] = None
    create: Optional[str] = None
    update: Optional[str] = None
    sevClient: Optional[SevClient] = None
    printCustomerNumber: Optional[bool] = None
    printContactPerson: Optional[bool] = None
    logoSize: Optional[int] = None
    color: Optional[str] = None
    
    class Config:
        populate_by_name = True


from typing import Optional
from pydantic import BaseModel, Field


class Export_Job_Download_Info(BaseModel):
    filename: Optional[str] = None
    link: Optional[str] = None
    linkExpireDate: Optional[str] = None
    class Config:
        populate_by_name = True


from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.contact import Contact
from sevdesk.converters.contactperson import ContactPerson
from sevdesk.converters.sevclient import SevClient
from sevdesk.converters.addresscountry import AddressCountry
from sevdesk.converters.createuser import CreateUser
from sevdesk.converters.taxrule import TaxRule
from sevdesk.converters.taxset import TaxSet
from sevdesk.converters.paymentmethod import PaymentMethod
from sevdesk.converters.origin import Origin

class Invoice(BaseModel):
    id_: Optional[int] = Field(default=None, alias="id")
    objectName: Optional[str] = None
    invoiceNumber: Optional[str] = None
    contact: Contact
    contactPerson: ContactPerson
    create: Optional[str] = None
    update: Optional[str] = None
    sevClient: Optional[SevClient] = None
    invoiceDate: str
    header: Optional[str] = None
    headText: Optional[str] = None
    footText: Optional[str] = None
    timeToPay: Optional[int] = None
    discount: int
    address: Optional[str] = None
    addressCountry: AddressCountry
    payDate: Optional[str] = None
    createUser: Optional[CreateUser] = None
    deliveryDate: Optional[str] = None
    deliveryDateUntil: Optional[int] = None
    status: str
    smallSettlement: Optional[bool] = None
    taxRate: float
    taxRule: TaxRule
    taxText: str
    taxType: str
    taxSet: Optional[TaxSet] = None
    dunningLevel: Optional[int] = None
    paymentMethod: Optional[PaymentMethod] = None
    sendDate: Optional[str] = None
    invoiceType: str
    accountIntervall: Optional[str] = None
    accountNextInvoice: Optional[int] = None
    currency: str
    sumNet: Optional[float] = None
    sumTax: Optional[float] = None
    sumGross: Optional[float] = None
    sumDiscounts: Optional[float] = None
    sumNetForeignCurrency: Optional[float] = None
    sumTaxForeignCurrency: Optional[float] = None
    sumGrossForeignCurrency: Optional[float] = None
    sumDiscountsForeignCurrency: Optional[float] = None
    sumNetAccounting: Optional[float] = None
    sumTaxAccounting: Optional[float] = None
    sumGrossAccounting: Optional[float] = None
    paidAmount: Optional[float] = None
    showNet: Optional[bool] = None
    enshrined: Optional[str] = None
    sendType: Optional[str] = None
    origin: Optional[Origin] = None
    customerInternalNote: Optional[str] = None
    propertyIsEInvoice: Optional[bool] = None
    mapAll: bool
    class Config:
        populate_by_name = True


from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.sevclient import SevClient

class Email(BaseModel):
    id_: Optional[int] = Field(default=None, alias="id")
    objectName: Optional[str] = None
    create: Optional[str] = None
    update: Optional[str] = None
    object_: Optional[str] = Field(default=None, alias="object")
    from_: str = Field(alias="from")
    to_: str = Field(alias="to")
    subject: str
    text: Optional[str] = None
    sevClient: Optional[SevClient] = None
    cc: Optional[str] = None
    bcc: Optional[str] = None
    arrived: Optional[str] = None
    class Config:
        populate_by_name = True


from sevdesk.base.basecontroller import BaseController
from typing import Optional

class ReportController(BaseController):

    @BaseController.get("/Report/invoicelist")
    def reportInvoice(self, sevQuery: dict, view: str, download: Optional[bool] = None):
        """Export invoice list"""
        return (yield)

    @BaseController.get("/Report/orderlist")
    def reportOrder(self, sevQuery: dict, view: str, download: Optional[bool] = None):
        """Export order list"""
        return (yield)

    @BaseController.get("/Report/contactlist")
    def reportContact(self, sevQuery: dict, download: Optional[bool] = None):
        """Export contact list"""
        return (yield)

    @BaseController.get("/Report/voucherlist")
    def reportVoucher(self, sevQuery: dict, download: Optional[bool] = None):
        """Export voucher list"""
        return (yield)




from sevdesk.base.basecontroller import BaseController
from typing import Optional
from sevdesk.models.export_job_download_info import Export_Job_Download_Info
from sevdesk.models.export_progress_data import Export_Progress_Data

class ExportController(BaseController):

    @BaseController.put("/SevClient/{SevClientId}/updateExportConfig")
    def updateExportConfig(self, SevClientId: float):
        """Update export config"""
        return (yield)

    @BaseController.get("/Export/datevCSV")
    def exportDatevDepricated(self, endDate: int, scope: str, startDate: int, Download: Optional[bool] = None, enshrine: Optional[bool] = None, withEnshrinedDocuments: Optional[bool] = None, withUnpaidDocuments: Optional[bool] = None):
        """Export datev"""
        return (yield)

    @BaseController.get("/Export/createDatevCsvZipExportJob")
    def exportDatevCSV(self, endDate: int, scope: str, startDate: int, enshrineDocuments: Optional[bool] = None, exportByPaydate: Optional[bool] = None, includeDocumentImages: Optional[bool] = None, includeEnshrined: Optional[bool] = None):
        """Start DATEV CSV ZIP export"""
        return (yield)

    @BaseController.get("/Export/createDatevXmlZipExportJob")
    def exportDatevXML(self, endDate: int, scope: str, startDate: int, exportByPaydate: Optional[bool] = None, includeDocumentXml: Optional[bool] = None, includeEnshrined: Optional[bool] = None, includeExportedDocuments: Optional[bool] = None):
        """Start DATEV XML ZIP export"""
        return (yield)

    @BaseController.get("/Progress/generateDownloadHash")
    def generateDownloadHash(self, jobId: str) -> list[Export_Progress_Data]:
        """Generate download hash"""
        return (yield)

    @BaseController.get("/Progress/getProgress")
    def getProgress(self, hash: str) -> list[Export_Progress_Data]:
        """Get progress"""
        return (yield)

    @BaseController.get("/ExportJob/jobDownloadInfo")
    def jobDownloadInfo(self, jobId: str) -> list[Export_Job_Download_Info]:
        """Get job download info"""
        return (yield)

    @BaseController.get("/Export/invoiceCsv")
    def exportInvoice(self, sevQuery: dict, download: Optional[bool] = None):
        """Export invoice"""
        return (yield)

    @BaseController.get("/Export/invoiceZip")
    def exportInvoiceZip(self, sevQuery: dict, download: Optional[bool] = None):
        """Export Invoice as zip"""
        return (yield)

    @BaseController.get("/Export/creditNoteCsv")
    def exportCreditNote(self, sevQuery: dict, download: Optional[bool] = None):
        """Export creditNote"""
        return (yield)

    @BaseController.get("/Export/voucherListCsv")
    def exportVoucher(self, sevQuery: dict, download: Optional[bool] = None):
        """Export voucher as zip"""
        return (yield)

    @BaseController.get("/Export/transactionsCsv")
    def exportTransactions(self, sevQuery: dict, download: Optional[bool] = None):
        """Export transaction"""
        return (yield)

    @BaseController.get("/Export/voucherZip")
    def exportVoucherZip(self, sevQuery: dict, download: Optional[bool] = None):
        """Export voucher zip"""
        return (yield)

    @BaseController.get("/Export/contactListCsv")
    def exportContact(self, sevQuery: dict, download: Optional[bool] = None):
        """Export contact"""
        return (yield)




from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.category import Category
from sevdesk.converters.unity import Unity
from sevdesk.converters.sevclient import SevClient

class PartUpdate(BaseModel):
    id_: Optional[int] = Field(default=None, alias="id")
    objectName: Optional[str] = None
    create: Optional[str] = None
    update: Optional[str] = None
    name: Optional[str] = None
    partNumber: Optional[str] = None
    text: Optional[str] = None
    category: Optional[Category] = None
    stock: Optional[float] = None
    stockEnabled: Optional[bool] = None
    unity: Optional[Unity] = None
    price: Optional[float] = None
    priceNet: Optional[float] = None
    priceGross: Optional[float] = None
    sevClient: Optional[SevClient] = None
    pricePurchase: Optional[float] = None
    taxRate: Optional[float] = None
    status: Optional[int] = None
    internalComment: Optional[str] = None
    class Config:
        populate_by_name = True


from sevdesk.base.basecontroller import BaseController
from typing import Optional
from sevdesk.models.accountingcontact import AccountingContact
from sevdesk.models.accountingcontactresponse import AccountingContactResponse
from sevdesk.models.accountingcontactupdate import AccountingContactUpdate

class AccountingContactController(BaseController):

    @BaseController.get("/AccountingContact")
    def getAccountingContact(self, contact_id: Optional[str] = None, contact_objectName: Optional[str] = None) -> list[AccountingContactResponse]:
        """Retrieve accounting contact"""
        return (yield)

    @BaseController.post("/AccountingContact")
    def createAccountingContact(self, body: AccountingContact) -> AccountingContactResponse:
        """Create a new accounting contact"""
        return (yield)

    @BaseController.get("/AccountingContact/{accountingContactId}")
    def getAccountingContactById(self, accountingContactId: int) -> list[AccountingContactResponse]:
        """Find accounting contact by ID"""
        return (yield)

    @BaseController.put("/AccountingContact/{accountingContactId}")
    def updateAccountingContact(self, accountingContactId: int, body: AccountingContactUpdate) -> AccountingContactResponse:
        """Update an existing accounting contact"""
        return (yield)

    @BaseController.delete("/AccountingContact/{accountingContactId}")
    def deleteAccountingContact(self, accountingContactId: int):
        """Deletes an accounting contact"""
        return (yield)




from sevdesk.base.basecontroller import BaseController
from typing import Optional
from sevdesk.models.contactcustomfield import ContactCustomField
from sevdesk.models.contactcustomfieldresponse import ContactCustomFieldResponse
from sevdesk.models.contactcustomfieldsetting import ContactCustomFieldSetting
from sevdesk.models.contactcustomfieldsettingresponse import ContactCustomFieldSettingResponse
from sevdesk.models.contactcustomfieldsettingupdate import ContactCustomFieldSettingUpdate
from sevdesk.models.contactcustomfieldupdate import ContactCustomFieldUpdate
from sevdesk.models.textparser_fetchdictionaryentriesbytype_response import Textparser_fetchDictionaryEntriesByType_response

class ContactFieldController(BaseController):

    @BaseController.get("/Textparser/fetchDictionaryEntriesByType")
    def getPlaceholder(self, objectName: str, subObjectName: Optional[str] = None) -> list[Textparser_fetchDictionaryEntriesByType_response]:
        """Retrieve Placeholders"""
        return (yield)

    @BaseController.get("/ContactCustomField")
    def getContactFields(self) -> list[ContactCustomFieldResponse]:
        """Retrieve contact fields"""
        return (yield)

    @BaseController.post("/ContactCustomField")
    def createContactField(self, body: ContactCustomField) -> ContactCustomFieldResponse:
        """Create contact field"""
        return (yield)

    @BaseController.get("/ContactCustomField/{contactCustomFieldId}")
    def getContactFieldsById(self, contactCustomFieldId: float) -> list[ContactCustomFieldResponse]:
        """Retrieve contact fields"""
        return (yield)

    @BaseController.put("/ContactCustomField/{contactCustomFieldId}")
    def updateContactfield(self, contactCustomFieldId: float, body: ContactCustomFieldUpdate) -> ContactCustomFieldResponse:
        """Update a contact field"""
        return (yield)

    @BaseController.delete("/ContactCustomField/{contactCustomFieldId}")
    def deleteContactCustomFieldId(self, contactCustomFieldId: int):
        """delete a contact field"""
        return (yield)

    @BaseController.get("/ContactCustomFieldSetting")
    def getContactFieldSettings(self) -> list[ContactCustomFieldSettingResponse]:
        """Retrieve contact field settings"""
        return (yield)

    @BaseController.post("/ContactCustomFieldSetting")
    def createContactFieldSetting(self, body: ContactCustomFieldSetting) -> list[ContactCustomFieldSettingResponse]:
        """Create contact field setting"""
        return (yield)

    @BaseController.get("/ContactCustomFieldSetting/{contactCustomFieldSettingId}")
    def getContactFieldSettingById(self, contactCustomFieldSettingId: int) -> list[ContactCustomFieldSettingResponse]:
        """Find contact field setting by ID"""
        return (yield)

    @BaseController.put("/ContactCustomFieldSetting/{contactCustomFieldSettingId}")
    def updateContactFieldSetting(self, contactCustomFieldSettingId: int, body: ContactCustomFieldSettingUpdate) -> ContactCustomFieldSettingResponse:
        """Update contact field setting"""
        return (yield)

    @BaseController.delete("/ContactCustomFieldSetting/{contactCustomFieldSettingId}")
    def deleteContactFieldSetting(self, contactCustomFieldSettingId: int):
        """Deletes a contact field setting"""
        return (yield)

    @BaseController.get("/ContactCustomFieldSetting/{contactCustomFieldSettingId}/getReferenceCount")
    def getReferenceCount(self, contactCustomFieldSettingId: int):
        """Receive count reference"""
        return (yield)




from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.metadaten import Metadaten

class ChangeLayoutResponse(BaseModel):
    result: Optional[str] = None
    metadaten: Optional[Metadaten] = None
    class Config:
        populate_by_name = True


from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.supplier import Supplier
from sevdesk.converters.taxrule import TaxRule
from sevdesk.converters.taxset import TaxSet
from sevdesk.converters.document import Document
from sevdesk.converters.costcentre import CostCentre

class VoucherUpdate(BaseModel):
    voucherDate: Optional[str] = None
    supplier: Optional[Supplier] = None
    supplierName: Optional[str] = None
    description: Optional[str] = None
    payDate: Optional[str] = None
    status: Optional[float] = None
    paidAmount: Optional[float] = None
    taxRule: Optional[TaxRule] = None
    taxType: Optional[str] = None
    creditDebit: Optional[str] = None
    voucherType: Optional[str] = None
    currency: Optional[str] = None
    propertyForeignCurrencyDeadline: Optional[str] = None
    propertyExchangeRate: Optional[float] = None
    taxSet: Optional[TaxSet] = None
    paymentDeadline: Optional[str] = None
    deliveryDate: Optional[str] = None
    deliveryDateUntil: Optional[str] = None
    document: Optional[Document] = None
    costCentre: Optional[CostCentre] = None
    class Config:
        populate_by_name = True


from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.sevclient import SevClient
from sevdesk.converters.voucher import Voucher
from sevdesk.converters.accountdatev import AccountDatev
from sevdesk.converters.accountingtype import AccountingType
from sevdesk.converters.estimatedaccountingtype import EstimatedAccountingType

class VoucherPos(BaseModel):
    id_: Optional[int] = Field(default=None, alias="id")
    objectName: str
    mapAll: bool
    create: Optional[str] = None
    update: Optional[str] = None
    sevClient: Optional[SevClient] = None
    voucher: Voucher
    accountDatev: AccountDatev
    accountingType: AccountingType
    estimatedAccountingType: Optional[EstimatedAccountingType] = None
    taxRate: float
    net: bool
    isAsset: Optional[bool] = None
    sumNet: float
    sumTax: Optional[float] = None
    sumGross: float
    sumNetAccounting: Optional[float] = None
    sumTaxAccounting: Optional[float] = None
    sumGrossAccounting: Optional[float] = None
    comment: Optional[str] = None
    class Config:
        populate_by_name = True


from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.contact import Contact
from sevdesk.converters.sevclient import SevClient
from sevdesk.converters.addresscountry import AddressCountry
from sevdesk.converters.createuser import CreateUser
from sevdesk.converters.contactperson import ContactPerson
from sevdesk.converters.taxrule import TaxRule
from sevdesk.converters.paymentmethod import PaymentMethod
from sevdesk.converters.costcentre import CostCentre
from sevdesk.converters.origin import Origin
from sevdesk.converters.taxset import TaxSet

class InvoiceResponse(BaseModel):
    id_: Optional[str] = Field(default=None, alias="id")
    objectName: Optional[str] = None
    invoiceNumber: Optional[str] = None
    contact: Optional[Contact] = None
    create: Optional[str] = None
    update: Optional[str] = None
    sevClient: Optional[SevClient] = None
    invoiceDate: Optional[str] = None
    header: Optional[str] = None
    headText: Optional[str] = None
    footText: Optional[str] = None
    timeToPay: Optional[str] = None
    discountTime: Optional[str] = None
    discount: Optional[str] = None
    addressCountry: Optional[AddressCountry] = None
    payDate: Optional[str] = None
    createUser: Optional[CreateUser] = None
    deliveryDate: Optional[str] = None
    status: Optional[str] = None
    smallSettlement: Optional[bool] = None
    contactPerson: Optional[ContactPerson] = None
    taxRate: Optional[str] = None
    taxRule: Optional[TaxRule] = None
    taxText: Optional[str] = None
    dunningLevel: Optional[str] = None
    taxType: Optional[str] = None
    paymentMethod: Optional[PaymentMethod] = None
    costCentre: Optional[CostCentre] = None
    sendDate: Optional[str] = None
    origin: Optional[Origin] = None
    invoiceType: Optional[str] = None
    accountIntervall: Optional[str] = None
    accountNextInvoice: Optional[str] = None
    reminderTotal: Optional[str] = None
    reminderDebit: Optional[str] = None
    reminderDeadline: Optional[str] = None
    reminderCharge: Optional[str] = None
    taxSet: Optional[TaxSet] = None
    address: Optional[str] = None
    currency: Optional[str] = None
    sumNet: Optional[str] = None
    sumTax: Optional[str] = None
    sumGross: Optional[str] = None
    sumDiscounts: Optional[str] = None
    sumNetForeignCurrency: Optional[str] = None
    sumTaxForeignCurrency: Optional[str] = None
    sumGrossForeignCurrency: Optional[str] = None
    sumDiscountsForeignCurrency: Optional[str] = None
    sumNetAccounting: Optional[str] = None
    sumTaxAccounting: Optional[str] = None
    sumGrossAccounting: Optional[str] = None
    paidAmount: Optional[float] = None
    customerInternalNote: Optional[str] = None
    showNet: Optional[bool] = None
    enshrined: Optional[str] = None
    sendType: Optional[str] = None
    deliveryDateUntil: Optional[str] = None
    datevConnectOnline: Optional[dict] = None
    sendPaymentReceivedNotificationDate: Optional[str] = None
    class Config:
        populate_by_name = True


from sevdesk.base.basecontroller import BaseController
from typing import Optional
from sevdesk.models.voucherposresponse import VoucherPosResponse

class VoucherPosController(BaseController):

    @BaseController.get("/VoucherPos")
    def getVoucherPositions(self, voucher_id: Optional[int] = None, voucher_objectName: Optional[str] = None) -> list[VoucherPosResponse]:
        """Retrieve voucher positions"""
        return (yield)




from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.contact import Contact
from sevdesk.converters.addresscountry import AddressCountry
from sevdesk.converters.createuser import CreateUser
from sevdesk.converters.sevclient import SevClient
from sevdesk.converters.contactperson import ContactPerson
from sevdesk.converters.taxrule import TaxRule
from sevdesk.converters.taxset import TaxSet

class CreditNote(BaseModel):
    id_: Optional[int] = Field(default=None, alias="id")
    objectName: str
    mapAll: bool
    create: Optional[str] = None
    update: Optional[str] = None
    creditNoteNumber: str
    contact: Contact
    creditNoteDate: str
    status: str
    header: str
    headText: Optional[str] = None
    footText: Optional[str] = None
    addressCountry: Optional[AddressCountry]
    createUser: Optional[CreateUser] = None
    sevClient: Optional[SevClient] = None
    smallSettlement: Optional[bool] = None
    contactPerson: ContactPerson
    taxRule: TaxRule
    taxRate: float
    taxSet: Optional[TaxSet] = None
    taxText: str
    taxType: str
    sendDate: Optional[str] = None
    address: Optional[str] = None
    bookingCategory: str
    currency: str
    sumNet: Optional[float] = None
    sumTax: Optional[float] = None
    sumGross: Optional[float] = None
    sumDiscounts: Optional[float] = None
    sumNetForeignCurrency: Optional[float] = None
    sumTaxForeignCurrency: Optional[float] = None
    sumGrossForeignCurrency: Optional[float] = None
    sumDiscountsForeignCurrency: Optional[float] = None
    customerInternalNote: Optional[str] = None
    showNet: Optional[bool] = None
    sendType: Optional[str] = None
    class Config:
        populate_by_name = True


from typing import Optional
from pydantic import BaseModel, Field


class Export_Progress_Data(BaseModel):
    current: Optional[int] = None
    total: Optional[int] = None
    class Config:
        populate_by_name = True


from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.order import Order

class CreateInvoiceFromOrder(BaseModel):
    order: Order
    type_: Optional[str] = Field(default=None, alias="type")
    amount: Optional[float] = None
    partialType: Optional[str] = None
    class Config:
        populate_by_name = True


from pydantic import BaseModel, Field

from sevdesk.converters.contact import Contact
from sevdesk.converters.contactcustomfieldsetting import ContactCustomFieldSetting

class ContactCustomField(BaseModel):
    contact: Contact
    contactCustomFieldSetting: ContactCustomFieldSetting
    value: str
    objectName: str
    class Config:
        populate_by_name = True


import inspect
from typing import get_type_hints, get_origin, get_args

class BaseController:
    def __init__(self, client):
        self.client = client
    
    @staticmethod
    def parse_response(response, return_type):
        """Wandelt die Response in das entsprechende Model um"""
        if return_type is None:
            return response
        
        # Hole den ursprünglichen Typ (ohne list[] wrapper)
        origin = get_origin(return_type)
        
        # Wenn es eine Liste ist
        if origin is list:
            args = get_args(return_type)
            if args:
                model_class = args[0]
                # Response sollte eine Liste von Dicts sein
                if isinstance(response, dict) and 'objects' in response:
                    # sevDesk API struktur: {"objects": [...]}
                    return [model_class(**item) if isinstance(item, dict) else item 
                            for item in response['objects']]
                elif isinstance(response, list):
                    return [model_class(**item) if isinstance(item, dict) else item 
                            for item in response]
        
        # Einzelnes Model
        elif hasattr(return_type, '__bases__'):  # Check if it's a class
            if isinstance(response, dict):
                # sevDesk API struktur kann auch {"objects": {...}} sein
                if 'objects' in response and isinstance(response['objects'], dict):
                    return return_type(**response['objects'])
                else:
                    return return_type(**response)
        
        return response
    
    @staticmethod
    def request(method, path):
        def decorator(func):
            def wrapper(self, *args, **kwargs):
                # Args in kwargs umwandeln
                sig = inspect.signature(func)
                bound_args = sig.bind(self, *args, **kwargs)
                bound_args.apply_defaults()
                
                # Entferne 'self' aus den arguments
                all_kwargs = dict(bound_args.arguments)
                all_kwargs.pop('self', None)
                
                gen = func(self, **all_kwargs)
                
                try:
                    # Pre-request: bis zum yield ausführen
                    next(gen)
                    # veränderte parameter übernehmen
                    for arg in all_kwargs.copy():
                        # gi_frame HACK
                        if gen.gi_frame:
                            all_kwargs[arg] = gen.gi_frame.f_locals[arg]
                    
                    response = self.client.request(method, path, all_kwargs)
                    
                    # Return Type aus der Funktion auslesen
                    try:
                        type_hints = get_type_hints(func)
                        return_type = type_hints.get('return', None)
                    except Exception:
                        return_type = None
                    
                    # Response in Model umwandeln
                    parsed_response = BaseController.parse_response(response, return_type)
                    
                    # Gebe das Ergebnis direkt zurück
                    return parsed_response
                    
                except StopIteration:
                    return None
                    
            wrapper.__name__ = func.__name__
            return wrapper
        return decorator

    @staticmethod
    def get(path):
        return BaseController.request('get', path)

    @staticmethod
    def post(path):
        return BaseController.request('post', path)

    @staticmethod
    def put(path):
        return BaseController.request('put', path)

    @staticmethod
    def delete(path):
        return BaseController.request('delete', path)


from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.contact import Contact
from sevdesk.converters.sevclient import SevClient
from sevdesk.converters.contactperson import ContactPerson
from sevdesk.converters.addresscountry import AddressCountry

class Letter(BaseModel):
    """Model zum Erstellen eines neuen Briefes"""
    contact: Contact
    letterDate: str
    header: str
    status: str
    contactPerson: ContactPerson
    text: str
    objectName: str = "Letter"
    address: Optional[str] = None
    addressCountry: Optional[AddressCountry] = None
    types: Optional[str] = None
    
    class Config:
        populate_by_name = True


from sevdesk.base.basecontroller import BaseController
from sevdesk.models.checkaccountresponse import CheckAccountResponse
from sevdesk.models.checkaccountupdate import CheckAccountUpdate
from sevdesk.models.createclearingaccount import CreateClearingAccount
from sevdesk.models.createclearingaccountresponse import CreateClearingAccountResponse
from sevdesk.models.createfileimportaccount import CreateFileImportAccount
from sevdesk.models.createfileimportaccountresponse import CreateFileImportAccountResponse

class CheckAccountController(BaseController):

    @BaseController.get("/CheckAccount")
    def getCheckAccounts(self) -> list[CheckAccountResponse]:
        """Retrieve check accounts"""
        return (yield)

    @BaseController.post("/CheckAccount/Factory/fileImportAccount")
    def createFileImportAccount(self, body: CreateFileImportAccount) -> CreateFileImportAccountResponse:
        """Create a new file import account"""
        return (yield)

    @BaseController.post("/CheckAccount/Factory/clearingAccount")
    def createClearingAccount(self, body: CreateClearingAccount) -> CreateClearingAccountResponse:
        """Create a new clearing account"""
        return (yield)

    @BaseController.get("/CheckAccount/{checkAccountId}")
    def getCheckAccountById(self, checkAccountId: int) -> list[CheckAccountResponse]:
        """Find check account by ID"""
        return (yield)

    @BaseController.put("/CheckAccount/{checkAccountId}")
    def updateCheckAccount(self, checkAccountId: int, body: CheckAccountUpdate) -> CheckAccountResponse:
        """Update an existing check account"""
        return (yield)

    @BaseController.delete("/CheckAccount/{checkAccountId}")
    def deleteCheckAccount(self, checkAccountId: int):
        """Deletes a check account"""
        return (yield)

    @BaseController.get("/CheckAccount/{checkAccountId}/getBalanceAtDate")
    def getBalanceAtDate(self, checkAccountId: int, date: str):
        """Get the balance at a given date"""
        return (yield)




from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.contact import Contact
from sevdesk.converters.addresscountry import AddressCountry
from sevdesk.converters.contactperson import ContactPerson
from sevdesk.converters.taxrule import TaxRule
from sevdesk.converters.taxset import TaxSet
from sevdesk.converters.origin import Origin

class Order(BaseModel):
    id_: Optional[int] = Field(default=None, alias="id")
    objectName: Optional[str] = None
    mapAll: bool
    create: Optional[str] = None
    update: Optional[str] = None
    orderNumber: str
    contact: Contact
    orderDate: str
    status: int
    header: str
    headText: Optional[str] = None
    footText: Optional[str] = None
    addressCountry: AddressCountry
    deliveryTerms: Optional[str] = None
    paymentTerms: Optional[str] = None
    version: int
    smallSettlement: Optional[bool] = None
    contactPerson: ContactPerson
    taxRate: float
    taxRule: TaxRule
    taxSet: Optional[TaxSet] = None
    taxText: str
    taxType: str
    orderType: Optional[str] = None
    sendDate: Optional[str] = None
    address: Optional[str] = None
    currency: str
    customerInternalNote: Optional[str] = None
    showNet: Optional[bool] = None
    sendType: Optional[str] = None
    origin: Optional[Origin] = None
    class Config:
        populate_by_name = True


from sevdesk.base.basecontroller import BaseController
from typing import Optional
from sevdesk.models.contact import Contact
from sevdesk.models.contactresponse import ContactResponse
from sevdesk.models.contactupdate import ContactUpdate

class ContactController(BaseController):

    @BaseController.get("/Contact/Factory/getNextCustomerNumber")
    def getNextCustomerNumber(self):
        """Get next free customer number"""
        return (yield)

    @BaseController.get("/Contact/Factory/findContactsByCustomFieldValue")
    def findContactsByCustomFieldValue(self, customFieldName: str, value: str, customFieldSetting_id: Optional[str] = None, customFieldSetting_objectName: Optional[str] = None) -> list[ContactResponse]:
        """Find contacts by custom field value"""
        return (yield)

    @BaseController.get("/Contact/Mapper/checkCustomerNumberAvailability")
    def contactCustomerNumberAvailabilityCheck(self, customerNumber: Optional[str] = None):
        """Check if a customer number is available"""
        return (yield)

    @BaseController.get("/Contact")
    def getContacts(self, customerNumber: Optional[str] = None, depth: Optional[str] = None) -> list[ContactResponse]:
        """Retrieve contacts"""
        return (yield)

    @BaseController.post("/Contact")
    def createContact(self, body: Contact) -> ContactResponse:
        """Create a new contact"""
        return (yield)

    @BaseController.get("/Contact/{contactId}")
    def getContactById(self, contactId: int) -> list[ContactResponse]:
        """Find contact by ID"""
        return (yield)

    @BaseController.put("/Contact/{contactId}")
    def updateContact(self, contactId: int, body: ContactUpdate) -> ContactResponse:
        """Update a existing contact"""
        return (yield)

    @BaseController.delete("/Contact/{contactId}")
    def deleteContact(self, contactId: int):
        """Deletes a contact"""
        return (yield)

    @BaseController.get("/Contact/{contactId}/getTabsItemCount")
    def getContactTabsItemCountById(self, contactId: int):
        """Get number of all items"""
        return (yield)




from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.contact import Contact

class AccountingContactUpdate(BaseModel):
    contact: Optional[Contact] = None
    debitorNumber: Optional[int] = None
    creditorNumber: Optional[int] = None
    class Config:
        populate_by_name = True


from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.sevclient import SevClient
from sevdesk.converters.createuser import CreateUser
from sevdesk.converters.supplier import Supplier
from sevdesk.converters.taxrule import TaxRule
from sevdesk.converters.taxset import TaxSet
from sevdesk.converters.document import Document
from sevdesk.converters.costcentre import CostCentre

class Voucher(BaseModel):
    id_: Optional[int] = Field(default=None, alias="id")
    objectName: str
    mapAll: bool
    create: Optional[str] = None
    update: Optional[str] = None
    sevClient: Optional[SevClient] = None
    createUser: Optional[CreateUser] = None
    voucherDate: Optional[str] = None
    supplier: Optional[Supplier] = None
    supplierName: Optional[str] = None
    description: Optional[str] = None
    payDate: Optional[str] = None
    status: float
    sumNet: Optional[float] = None
    sumTax: Optional[float] = None
    sumGross: Optional[float] = None
    sumNetAccounting: Optional[float] = None
    sumTaxAccounting: Optional[float] = None
    sumGrossAccounting: Optional[float] = None
    sumDiscounts: Optional[float] = None
    sumDiscountsForeignCurrency: Optional[float] = None
    paidAmount: Optional[float] = None
    taxRule: TaxRule
    taxType: str
    creditDebit: str
    voucherType: str
    currency: Optional[str] = None
    propertyForeignCurrencyDeadline: Optional[str] = None
    propertyExchangeRate: Optional[float] = None
    recurringInterval: Optional[str] = None
    recurringStartDate: Optional[str] = None
    recurringNextVoucher: Optional[str] = None
    recurringLastVoucher: Optional[str] = None
    recurringEndDate: Optional[str] = None
    enshrined: Optional[str] = None
    taxSet: Optional[TaxSet] = None
    paymentDeadline: Optional[str] = None
    deliveryDate: Optional[str] = None
    deliveryDateUntil: Optional[str] = None
    document: Optional[Document] = None
    costCentre: Optional[CostCentre] = None
    class Config:
        populate_by_name = True


from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.parent import Parent
from sevdesk.converters.category import Category
from sevdesk.converters.sevclient import SevClient

class ContactResponse(BaseModel):
    id_: Optional[str] = Field(default=None, alias="id")
    objectName: Optional[str] = None
    create: Optional[str] = None
    update: Optional[str] = None
    name: Optional[str] = None
    status: Optional[str] = None
    customerNumber: Optional[str] = None
    parent: Optional[Parent] = None
    surename: Optional[str] = None
    familyname: Optional[str] = None
    titel: Optional[str] = None
    category: Optional[Category] = None
    description: Optional[str] = None
    academicTitle: Optional[str] = None
    gender: Optional[str] = None
    sevClient: Optional[SevClient] = None
    name2: Optional[str] = None
    birthday: Optional[str] = None
    vatNumber: Optional[str] = None
    bankAccount: Optional[str] = None
    bankNumber: Optional[str] = None
    defaultCashbackTime: Optional[str] = None
    defaultCashbackPercent: Optional[str] = None
    defaultTimeToPay: Optional[str] = None
    taxNumber: Optional[str] = None
    taxOffice: Optional[str] = None
    exemptVat: Optional[str] = None
    defaultDiscountAmount: Optional[str] = None
    defaultDiscountPercentage: Optional[str] = None
    buyerReference: Optional[str] = None
    governmentAgency: Optional[str] = None
    additionalInformation: Optional[str] = None
    class Config:
        populate_by_name = True


from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.sevclient import SevClient
from sevdesk.converters.voucher import Voucher
from sevdesk.converters.accountdatev import AccountDatev
from sevdesk.converters.accountingtype import AccountingType
from sevdesk.converters.estimatedaccountingtype import EstimatedAccountingType

class VoucherPosResponse(BaseModel):
    id_: Optional[str] = Field(default=None, alias="id")
    objectName: Optional[str] = None
    create: Optional[str] = None
    update: Optional[str] = None
    sevClient: Optional[SevClient] = None
    voucher: Voucher
    accountDatev: AccountDatev
    accountingType: AccountingType
    estimatedAccountingType: Optional[EstimatedAccountingType] = None
    taxRate: str
    net: bool
    isAsset: Optional[bool] = None
    sumNet: str
    sumTax: Optional[str] = None
    sumGross: str
    sumNetAccounting: Optional[str] = None
    sumTaxAccounting: Optional[str] = None
    sumGrossAccounting: Optional[str] = None
    comment: Optional[str] = None
    class Config:
        populate_by_name = True


from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.contact import Contact
from sevdesk.converters.addresscountry import AddressCountry
from sevdesk.converters.createuser import CreateUser
from sevdesk.converters.sevclient import SevClient
from sevdesk.converters.origin import Origin
from sevdesk.converters.contactperson import ContactPerson
from sevdesk.converters.taxrule import TaxRule
from sevdesk.converters.taxset import TaxSet

class OrderResponse(BaseModel):
    id_: Optional[str] = Field(default=None, alias="id")
    objectName: Optional[str] = None
    create: Optional[str] = None
    update: Optional[str] = None
    orderNumber: Optional[str] = None
    contact: Optional[Contact] = None
    orderDate: Optional[str] = None
    status: Optional[str] = None
    header: Optional[str] = None
    headText: Optional[str] = None
    footText: Optional[str] = None
    addressCountry: Optional[AddressCountry] = None
    createUser: Optional[CreateUser] = None
    sevClient: Optional[SevClient] = None
    deliveryTerms: Optional[str] = None
    paymentTerms: Optional[str] = None
    origin: Optional[Origin] = None
    version: Optional[str] = None
    smallSettlement: Optional[bool] = None
    contactPerson: Optional[ContactPerson] = None
    taxRate: Optional[str] = None
    taxRule: Optional[TaxRule] = None
    taxSet: Optional[TaxSet] = None
    taxText: Optional[str] = None
    taxType: Optional[str] = None
    orderType: Optional[str] = None
    sendDate: Optional[str] = None
    address: Optional[str] = None
    currency: Optional[str] = None
    sumNet: Optional[str] = None
    sumTax: Optional[str] = None
    sumGross: Optional[str] = None
    sumDiscounts: Optional[str] = None
    sumNetForeignCurrency: Optional[str] = None
    sumTaxForeignCurrency: Optional[str] = None
    sumGrossForeignCurrency: Optional[str] = None
    sumDiscountsForeignCurrency: Optional[str] = None
    customerInternalNote: Optional[str] = None
    showNet: Optional[bool] = None
    sendType: Optional[str] = None
    class Config:
        populate_by_name = True


from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.contact import Contact
from sevdesk.converters.country import Country
from sevdesk.converters.category import Category

class ContactAddressUpdate(BaseModel):
    contact: Optional[Contact] = None
    street: Optional[str] = None
    zip: Optional[str] = None
    city: Optional[str] = None
    country: Optional[Country] = None
    category: Optional[Category] = None
    name: Optional[str] = None
    name2: Optional[str] = None
    name3: Optional[str] = None
    name4: Optional[str] = None
    class Config:
        populate_by_name = True


from typing import Optional, Any
from pydantic import BaseModel, Field


class SaveOrderResponse(BaseModel):
    order: Optional[str] = None
    orderPos: Optional[Any] = None
    class Config:
        populate_by_name = True


from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.invoice import Invoice
from sevdesk.converters.part import Part
from sevdesk.converters.unity import Unity
from sevdesk.converters.sevclient import SevClient

class InvoicePosResponse(BaseModel):
    id_: Optional[str] = Field(default=None, alias="id")
    objectName: Optional[str] = None
    create: Optional[str] = None
    update: Optional[str] = None
    invoice: Optional[Invoice] = None
    part: Optional[Part] = None
    quantity: Optional[bool] = None
    price: Optional[str] = None
    name: Optional[str] = None
    unity: Optional[Unity] = None
    sevClient: Optional[SevClient] = None
    positionNumber: Optional[str] = None
    text: Optional[str] = None
    discount: Optional[str] = None
    taxRate: Optional[str] = None
    sumDiscount: Optional[str] = None
    sumNetAccounting: Optional[str] = None
    sumTaxAccounting: Optional[str] = None
    sumGrossAccounting: Optional[str] = None
    priceNet: Optional[str] = None
    priceGross: Optional[str] = None
    priceTax: Optional[str] = None
    class Config:
        populate_by_name = True


from typing import Optional
from pydantic import BaseModel, Field


class CreditNote_mailResponse(BaseModel):
    id_: Optional[int] = Field(default=None, alias="id")
    objectName: Optional[str] = None
    additionalInformation: Optional[str] = None
    create: Optional[str] = None
    update: Optional[str] = None
    object_: Optional[str] = Field(default=None, alias="object")
    from_: Optional[str] = Field(default=None, alias="from")
    to_: Optional[str] = Field(default=None, alias="to")
    subject: Optional[str] = None
    text: Optional[str] = None
    sevClient: Optional[str] = None
    class Config:
        populate_by_name = True


from sevdesk.base.basecontroller import BaseController
from typing import Optional
from sevdesk.models.part import Part
from sevdesk.models.partupdate import PartUpdate

class PartController(BaseController):

    @BaseController.get("/Part")
    def getParts(self, name: Optional[str] = None, partNumber: Optional[str] = None) -> list[Part]:
        """Retrieve parts"""
        return (yield)

    @BaseController.post("/Part")
    def createPart(self, body: Part) -> Part:
        """Create a new part"""
        return (yield)

    @BaseController.get("/Part/{partId}")
    def getPartById(self, partId: int) -> list[Part]:
        """Find part by ID"""
        return (yield)

    @BaseController.put("/Part/{partId}")
    def updatePart(self, partId: int, body: PartUpdate) -> Part:
        """Update an existing part"""
        return (yield)

    @BaseController.get("/Part/{partId}/getStock")
    def partGetStock(self, partId: int):
        """Get stock of a part"""
        return (yield)




from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.contact import Contact
from sevdesk.converters.contactcustomfieldsetting import ContactCustomFieldSetting

class ContactCustomFieldUpdate(BaseModel):
    contact: Optional[Contact] = None
    contactCustomFieldSetting: Optional[ContactCustomFieldSetting] = None
    value: Optional[str] = None
    objectName: Optional[str] = None
    class Config:
        populate_by_name = True


from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.object_ import Object_

class Discount(BaseModel):
    id_: Optional[str] = Field(default=None, alias="id")
    objectName: Optional[str] = None
    create: Optional[str] = None
    update: Optional[str] = None
    object_: Optional[Object_] = Field(default=None, alias="object")
    sevClient: Optional[str] = None
    text: Optional[str] = None
    percentage: Optional[str] = None
    value: Optional[str] = None
    isNet: Optional[str] = None
    class Config:
        populate_by_name = True


import requests
import re
import importlib
from pathlib import Path

class Dummy:
    pass

class Client:

    def __init__(self, api_token, api_base='https://my.sevdesk.de/api/v1', session=None):
        self.api_token = api_token
        self.api_base = api_base
        self.session = session
        if not self.session:
            self.session = requests.Session()

        # Automatisch alle Controller laden
        controllers_dir = Path(__file__).parent / "controllers"
        self._load_controllers(controllers_dir, self, "sevdesk.controllers")
        
        self.undocumented = Dummy()
        controllers_dir2 = Path(__file__).parent / "undocumented" / "controllers"
        self._load_controllers(controllers_dir2, self.undocumented, "sevdesk.undocumented.controllers")


    def _load_controllers(self, controllers_dir, target, module_path):
        """Lädt automatisch alle Controller aus dem controllers Verzeichnis"""
        # controllers_dir = Path(__file__).parent / "controllers"
        
        if not controllers_dir.exists():
            return
        
        for controller_file in controllers_dir.glob("*_controller.py"):
            # z.B. "contact_controller.py" -> "contact"
            controller_name = controller_file.stem.replace("_controller", "")
            
            try:
                # Dynamisch importieren
                module = importlib.import_module(f"{module_path}.{controller_file.stem}")
                
                # Finde die Controller-Klasse im Modul (endet mit "Controller")
                controller_class = None
                for attr_name in dir(module):
                    attr = getattr(module, attr_name)
                    if (isinstance(attr, type) and 
                        attr_name.endswith("Controller") and 
                        attr_name != "BaseController"):
                        controller_class = attr
                        break
                
                if controller_class:
                    # Als Attribut setzen: self.contact = ContactController(self)
                    setattr(target, controller_name, controller_class(self))
                else:
                    print(f"Warning: No controller class found in {controller_file.stem}")
                
            except (ImportError, AttributeError) as e:
                print(f"Warning: Could not load controller {controller_name}: {e}")

    def request(self, method, path, params):
        url_params = re.findall(r"{(\w+)}", path)
        request_path = path.format(**params)
        
        request_params = {
            k: v for k, v in params.items()
            if k not in url_params and
            k != 'body' and
            v} # v not None
        request_url = f'{self.api_base}{request_path}'
        request_body = params.get('body', None)
        if request_body:
            request_body = request_body.model_dump(by_alias=True, exclude_none=True)

        # print('request_params', request_params)
        # print('request_url', request_url)
        # print('request_body', request_body)

        response = self.session.request(
            method=method,
            url=request_url,
            params=request_params,
            json=request_body,
            headers={
                'Authorization': self.api_token
            }
        )
        # print(response.status_code)
        # print(response.text)
        content_type = response.headers.get('content-type', '').lower()
        
        # PDF oder andere Binary-Daten
        if 'application/pdf' in content_type:
            return response.content
        
        # Andere Binary-Formate
        elif any(binary_type in content_type for binary_type in [
            'application/octet-stream',
            'image/',
            'application/zip',
            'application/xml'  # Für invoiceGetXml
        ]):
            return response.content
        else:
            return response.json()


from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.contact import Contact
from sevdesk.converters.contactperson import ContactPerson

class LetterUpdate(BaseModel):
    """Model zum Aktualisieren eines Briefes"""
    contact: Optional[Contact] = None
    letterDate: Optional[str] = None
    header: Optional[str] = None
    status: Optional[str] = None
    contactPerson: Optional[ContactPerson] = None
    text: Optional[str] = None
    sendDate: Optional[str] = None
    sendType: Optional[str] = None
    objectName: Optional[str] = "Letter"
    address: Optional[str] = None
    
    class Config:
        populate_by_name = True


from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.checkaccount import CheckAccount
from sevdesk.converters.sourcetransaction import SourceTransaction
from sevdesk.converters.targettransaction import TargetTransaction

class CheckAccountTransactionUpdate(BaseModel):
    valueDate: Optional[str] = None
    entryDate: Optional[str] = None
    paymtPurpose: Optional[str] = None
    amount: Optional[float] = None
    payeePayerName: Optional[str] = None
    checkAccount: Optional[CheckAccount] = None
    status: Optional[int] = None
    sourceTransaction: Optional[SourceTransaction] = None
    targetTransaction: Optional[TargetTransaction] = None
    class Config:
        populate_by_name = True


from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.contact import Contact
from sevdesk.converters.country import Country
from sevdesk.converters.category import Category
from sevdesk.converters.sevclient import SevClient

class ContactAddress(BaseModel):
    id_: Optional[int] = Field(default=None, alias="id")
    objectName: Optional[str] = None
    create: Optional[str] = None
    update: Optional[str] = None
    contact: Contact
    street: Optional[str] = None
    zip: Optional[str] = None
    city: Optional[str] = None
    country: Country
    category: Optional[Category]
    name: Optional[str] = None
    sevClient: Optional[SevClient] = None
    name2: Optional[str] = None
    name3: Optional[str] = None
    name4: Optional[str] = None
    class Config:
        populate_by_name = True


from typing import Optional
from pydantic import BaseModel, Field


class CheckAccountUpdate(BaseModel):
    name: Optional[str] = None
    type_: Optional[str] = Field(default=None, alias="type")
    importType: Optional[str] = None
    currency: Optional[str] = None
    defaultAccount: Optional[int] = None
    status: Optional[int] = None
    autoMapTransactions: Optional[int] = None
    accountingNumber: Optional[str] = None
    class Config:
        populate_by_name = True


from typing import Optional, Any
from pydantic import BaseModel, Field


class SaveVoucher(BaseModel):
    voucher: str
    voucherPosSave: Optional[Any] = None
    voucherPosDelete: Optional[str] = None
    filename: Optional[str] = None
    class Config:
        populate_by_name = True


from typing import Optional, Any
from pydantic import BaseModel, Field


class Textparser_fetchDictionaryEntriesByType_response(BaseModel):
    key: Optional[str] = None
    value: Optional[Any] = None
    class Config:
        populate_by_name = True


from sevdesk.base.basecontroller import BaseController
from typing import Optional
from sevdesk.models.receiptguidedto import ReceiptGuideDto
from sevdesk.models.savevoucher import SaveVoucher
from sevdesk.models.savevoucherresponse import SaveVoucherResponse
from sevdesk.models.voucherresponse import VoucherResponse
from sevdesk.models.voucherupdate import VoucherUpdate

class VoucherController(BaseController):

    @BaseController.post("/Voucher/Factory/saveVoucher")
    def voucherFactorySaveVoucher(self, body: SaveVoucher) -> SaveVoucherResponse:
        """Create a new voucher"""
        return (yield)

    @BaseController.post("/Voucher/Factory/uploadTempFile")
    def voucherUploadFile(self):
        """Upload voucher file"""
        return (yield)

    @BaseController.get("/Voucher")
    def getVouchers(self, contact_id: Optional[int] = None, contact_objectName: Optional[str] = None, creditDebit: Optional[str] = None, descriptionLike: Optional[str] = None, endDate: Optional[int] = None, startDate: Optional[int] = None, status: Optional[float] = None) -> list[VoucherResponse]:
        """Retrieve vouchers"""
        return (yield)

    @BaseController.get("/Voucher/{voucherId}")
    def getVoucherById(self, voucherId: int) -> list[VoucherResponse]:
        """Find voucher by ID"""
        return (yield)

    @BaseController.put("/Voucher/{voucherId}")
    def updateVoucher(self, voucherId: int, body: VoucherUpdate) -> VoucherResponse:
        """Update an existing voucher"""
        return (yield)

    @BaseController.put("/Voucher/{voucherId}/enshrine")
    def voucherEnshrine(self, voucherId: int):
        """Enshrine"""
        return (yield)

    @BaseController.put("/Voucher/{voucherId}/bookAmount")
    def bookVoucher(self, voucherId: int):
        """Book a voucher"""
        return (yield)

    @BaseController.put("/Voucher/{voucherId}/resetToOpen")
    def voucherResetToOpen(self, voucherId: int):
        """Reset status to open"""
        return (yield)

    @BaseController.put("/Voucher/{voucherId}/resetToDraft")
    def voucherResetToDraft(self, voucherId: int):
        """Reset status to draft"""
        return (yield)

    @BaseController.get("/ReceiptGuidance/forAllAccounts")
    def forAllAccounts(self) -> list[ReceiptGuideDto]:
        """Get all account guides"""
        return (yield)

    @BaseController.get("/ReceiptGuidance/forAccountNumber")
    def forAccountNumber(self, accountNumber: int) -> list[ReceiptGuideDto]:
        """Get guidance by account number"""
        return (yield)

    @BaseController.get("/ReceiptGuidance/forTaxRule")
    def forTaxRule(self, taxRule: str) -> list[ReceiptGuideDto]:
        """Get guidance by Tax Rule"""
        return (yield)

    @BaseController.get("/ReceiptGuidance/forRevenue")
    def forRevenue(self) -> list[ReceiptGuideDto]:
        """Get guidance for revenue accounts"""
        return (yield)

    @BaseController.get("/ReceiptGuidance/forExpense")
    def forExpense(self) -> list[ReceiptGuideDto]:
        """Get guidance for expense accounts"""
        return (yield)




from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.creditnote import CreditNote
from sevdesk.converters.part import Part
from sevdesk.converters.unity import Unity
from sevdesk.converters.sevclient import SevClient

class CreditNotePosResponse(BaseModel):
    id_: Optional[str] = Field(default=None, alias="id")
    objectName: Optional[str] = None
    create: Optional[str] = None
    update: Optional[str] = None
    creditNote: CreditNote
    part: Optional[Part] = None
    quantity: str
    price: Optional[str] = None
    priceNet: Optional[str] = None
    priceTax: Optional[str] = None
    priceGross: Optional[str] = None
    name: Optional[str] = None
    unity: Unity
    sevClient: Optional[SevClient] = None
    positionNumber: Optional[str] = None
    text: Optional[str] = None
    discount: Optional[str] = None
    optional: Optional[bool] = None
    taxRate: str
    sumDiscount: Optional[str] = None
    class Config:
        populate_by_name = True


from typing import Optional
from pydantic import BaseModel, Field

class DiscountDelete(BaseModel):
    id_: Optional[int] = Field(default=None, alias="id")
    objectName: Optional[str] = None
    class Config:
        populate_by_name = True


from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.parent import Parent
from sevdesk.converters.category import Category

class Contact(BaseModel):
    name: Optional[str] = None
    status: Optional[int] = None
    customerNumber: Optional[str] = None
    parent: Optional[Parent] = None
    surename: Optional[str] = None
    familyname: Optional[str] = None
    titel: Optional[str] = None
    category: Category
    description: Optional[str] = None
    academicTitle: Optional[str] = None
    gender: Optional[str] = None
    name2: Optional[str] = None
    birthday: Optional[str] = None
    vatNumber: Optional[str] = None
    bankAccount: Optional[str] = None
    bankNumber: Optional[str] = None
    defaultCashbackTime: Optional[int] = None
    defaultCashbackPercent: Optional[float] = None
    defaultTimeToPay: Optional[int] = None
    taxNumber: Optional[str] = None
    taxOffice: Optional[str] = None
    exemptVat: Optional[bool] = None
    defaultDiscountAmount: Optional[float] = None
    defaultDiscountPercentage: Optional[bool] = None
    buyerReference: Optional[str] = None
    governmentAgency: Optional[bool] = None
    class Config:
        populate_by_name = True


from typing import Optional
from pydantic import BaseModel, Field


class ContactCustomFieldSetting(BaseModel):
    name: str
    description: Optional[str] = None
    objectName: Optional[str] = None
    class Config:
        populate_by_name = True


from sevdesk.base.basecontroller import BaseController
from typing import Optional
from sevdesk.models.changelayout import ChangeLayout
from sevdesk.models.changelayoutresponse import ChangeLayoutResponse

class LayoutController(BaseController):

    @BaseController.get("/DocServer/getLetterpapersWithThumb")
    def getLetterpapersWithThumb(self):
        """Retrieve letterpapers"""
        return (yield)

    @BaseController.get("/DocServer/getTemplatesWithThumb")
    def getTemplates(self, type_: Optional[str] = None):
        """Retrieve templates"""
        return (yield)

    @BaseController.put("/Invoice/{invoiceId}/changeParameter")
    def updateInvoiceTemplate(self, invoiceId: int, body: ChangeLayout) -> ChangeLayoutResponse:
        """Update an invoice template"""
        return (yield)

    @BaseController.put("/Order/{orderId}/changeParameter")
    def updateOrderTemplate(self, orderId: int, body: ChangeLayout) -> ChangeLayoutResponse:
        """Update an order template"""
        return (yield)

    @BaseController.put("/CreditNote/{creditNoteId}/changeParameter")
    def updateCreditNoteTemplate(self, creditNoteId: int, body: ChangeLayout) -> ChangeLayoutResponse:
        """Update an of credit note template"""
        return (yield)




from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.contact import Contact
from sevdesk.converters.sevclient import SevClient

class AccountingContactResponse(BaseModel):
    id_: Optional[str] = Field(default=None, alias="id")
    objectName: Optional[str] = None
    create: Optional[str] = None
    update: Optional[str] = None
    contact: Optional[Contact] = None
    sevClient: Optional[SevClient] = None
    debitorNumber: Optional[str] = None
    creditorNumber: Optional[str] = None
    class Config:
        populate_by_name = True


from sevdesk.base.basecontroller import BaseController
from sevdesk.models.contactaddress import ContactAddress
from sevdesk.models.contactaddressresponse import ContactAddressResponse
from sevdesk.models.contactaddressupdate import ContactAddressUpdate

class ContactAddressController(BaseController):

    @BaseController.post("/ContactAddress")
    def createContactAddress(self, body: ContactAddress) -> ContactAddressResponse:
        """Create a new contact address"""
        return (yield)

    @BaseController.get("/ContactAddress")
    def getContactAddresses(self) -> list[ContactAddressResponse]:
        """Retrieve contact addresses"""
        return (yield)

    @BaseController.get("/ContactAddress/{contactAddressId}")
    def contactAddressId(self, contactAddressId: int) -> list[ContactAddressResponse]:
        """Find contact address by ID"""
        return (yield)

    @BaseController.put("/ContactAddress/{contactAddressId}")
    def updateContactAddress(self, contactAddressId: int, body: ContactAddressUpdate) -> ContactAddressResponse:
        """update a existing contact address"""
        return (yield)

    @BaseController.delete("/ContactAddress/{contactAddressId}")
    def deleteContactAddress(self, contactAddressId: int):
        """Deletes a contact address"""
        return (yield)




from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.order import Order
from sevdesk.converters.part import Part
from sevdesk.converters.unity import Unity
from sevdesk.converters.sevclient import SevClient

class OrderPosUpdate(BaseModel):
    id_: Optional[int] = Field(default=None, alias="id")
    objectName: Optional[str] = None
    create: Optional[str] = None
    update: Optional[str] = None
    order: Optional[Order] = None
    part: Optional[Part] = None
    quantity: Optional[float] = None
    price: Optional[float] = None
    priceNet: Optional[float] = None
    priceTax: Optional[float] = None
    priceGross: Optional[float] = None
    name: Optional[str] = None
    unity: Optional[Unity] = None
    sevClient: Optional[SevClient] = None
    positionNumber: Optional[int] = None
    text: Optional[str] = None
    discount: Optional[float] = None
    optional: Optional[bool] = None
    taxRate: Optional[float] = None
    sumDiscount: Optional[float] = None
    class Config:
        populate_by_name = True


from sevdesk.base.basecontroller import BaseController
from typing import Optional, Any
from sevdesk.models.createpackinglistfromorder import CreatePackingListFromOrder
from sevdesk.models.discount import Discount
from sevdesk.models.emailorder import EmailOrder
from sevdesk.models.orderposresponse import OrderPosResponse
from sevdesk.models.orderresponse import OrderResponse
from sevdesk.models.orderupdate import OrderUpdate
from sevdesk.models.saveorder import SaveOrder
from sevdesk.models.saveorderresponse import SaveOrderResponse

class OrderController(BaseController):

    @BaseController.get("/Order")
    def getOrders(self, contact_id: Optional[int] = None, contact_objectName: Optional[str] = None, endDate: Optional[int] = None, orderNumber: Optional[str] = None, startDate: Optional[int] = None, status: Optional[int] = None) -> list[OrderResponse]:
        """Retrieve orders"""
        return (yield)

    @BaseController.post("/Order/Factory/saveOrder")
    def createOrder(self, body: SaveOrder) -> SaveOrderResponse:
        """Create a new order"""
        return (yield)

    @BaseController.get("/Order/{orderId}")
    def getOrderById(self, orderId: int) -> list[OrderResponse]:
        """Find order by ID"""
        return (yield)

    @BaseController.put("/Order/{orderId}")
    def updateOrder(self, orderId: int, body: OrderUpdate) -> OrderResponse:
        """Update an existing order"""
        return (yield)

    @BaseController.delete("/Order/{orderId}")
    def deleteOrder(self, orderId: int):
        """Deletes an order"""
        return (yield)

    @BaseController.get("/Order/{orderId}/getPositions")
    def getOrderPositionsById(self, orderId: int, embed: Optional[Any] = None, limit: Optional[int] = None, offset: Optional[int] = None) -> list[OrderPosResponse]:
        """Find order positions"""
        return (yield)

    @BaseController.get("/Order/{orderId}/getDiscounts")
    def getDiscounts(self, orderId: int, embed: Optional[Any] = None, limit: Optional[int] = None, offset: Optional[int] = None) -> list[Discount]:
        """Find order discounts"""
        return (yield)

    @BaseController.get("/Order/{orderId}/getRelatedObjects")
    def getRelatedObjects(self, orderId: int, embed: Optional[Any] = None, includeItself: Optional[bool] = None, sortByType: Optional[bool] = None) -> list[OrderPosResponse]:
        """Find related objects"""
        return (yield)

    @BaseController.post("/Order/{orderId}/sendViaEmail")
    def sendorderViaEMail(self, orderId: int) -> list[EmailOrder]:
        """Send order via email"""
        return (yield)

    @BaseController.post("/Order/Factory/createPackingListFromOrder")
    def createPackingListFromOrder(self, order_id: int, order_objectName: str, body: CreatePackingListFromOrder) -> OrderResponse:
        """Create packing list from order"""
        return (yield)

    @BaseController.post("/Order/Factory/createContractNoteFromOrder")
    def createContractNoteFromOrder(self, order_id: int, order_objectName: str, body: CreatePackingListFromOrder) -> OrderResponse:
        """Create contract note from order"""
        return (yield)

    @BaseController.get("/Order/{orderId}/getPdf")
    def orderGetPdf(self, orderId: int, download: Optional[bool] = None, preventSendBy: Optional[bool] = None):
        """Retrieve pdf document of an order"""
        return (yield)

    @BaseController.put("/Order/{orderId}/sendBy")
    def orderSendBy(self, orderId: int) -> OrderResponse:
        """Mark order as sent"""
        return (yield)




from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.tag import Tag
from sevdesk.converters.object_ import Object_
from sevdesk.converters.sevclient import SevClient

class TagCreateResponse(BaseModel):
    id_: Optional[str] = Field(default=None, alias="id")
    objectName: Optional[str] = None
    additionalInformation: Optional[str] = None
    create: Optional[str] = None
    tag: Optional[Tag] = None
    object_: Optional[Object_] = Field(default=None, alias="object")
    sevClient: Optional[SevClient] = None
    class Config:
        populate_by_name = True


from typing import Optional
from pydantic import BaseModel, Field


class CreateClearingAccount(BaseModel):
    name: Optional[str] = None
    accountingNumber: Optional[int] = None
    class Config:
        populate_by_name = True


from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.sevclient import SevClient

class ContactCustomFieldSettingResponse(BaseModel):
    id_: Optional[str] = Field(default=None, alias="id")
    objectName: Optional[str] = None
    create: Optional[str] = None
    update: Optional[str] = None
    sevClient: Optional[SevClient] = None
    name: Optional[str] = None
    identifier: Optional[str] = None
    description: Optional[str] = None
    class Config:
        populate_by_name = True


from sevdesk.base.basecontroller import BaseController
from typing import Optional
from sevdesk.models.tagcreateresponse import TagCreateResponse
from sevdesk.models.tagresponse import TagResponse

class TagController(BaseController):

    @BaseController.get("/Tag")
    def getTags(self, id_: Optional[float] = None, name: Optional[str] = None) -> list[TagResponse]:
        """Retrieve tags"""
        return (yield)

    @BaseController.get("/Tag/{tagId}")
    def getTagById(self, tagId: int) -> list[TagResponse]:
        """Find tag by ID"""
        return (yield)

    @BaseController.put("/Tag/{tagId}")
    def updateTag(self, tagId: int) -> TagResponse:
        """Update tag"""
        return (yield)

    @BaseController.delete("/Tag/{tagId}")
    def deleteTag(self, tagId: int):
        """Deletes a tag"""
        return (yield)

    @BaseController.post("/Tag/Factory/create")
    def createTag(self) -> TagCreateResponse:
        """Create a new tag"""
        return (yield)

    @BaseController.get("/TagRelation")
    def getTagRelations(self) -> list[TagCreateResponse]:
        """Retrieve tag relations"""
        return (yield)




from sevdesk.base.basecontroller import BaseController

class BasicsController(BaseController):

    @BaseController.get("/Tools/bookkeepingSystemVersion")
    def bookkeepingSystemVersion(self):
        """Retrieve bookkeeping system version"""
        return (yield)




from sevdesk.base.basecontroller import BaseController
from typing import Optional
from sevdesk.models.creditnoteposresponse import CreditNotePosResponse

class CreditNotePosController(BaseController):

    @BaseController.get("/CreditNotePos")
    def getcreditNotePositions(self, creditNote_id: Optional[int] = None, creditNote_objectName: Optional[str] = None) -> list[CreditNotePosResponse]:
        """Retrieve creditNote positions"""
        return (yield)




from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.contact import Contact
from sevdesk.converters.key import Key
from sevdesk.converters.sevclient import SevClient

class CommunicationWayResponse(BaseModel):
    id_: Optional[str] = Field(default=None, alias="id")
    objectName: Optional[str] = None
    create: Optional[str] = None
    update: Optional[str] = None
    contact: Optional[Contact] = None
    type_: Optional[str] = Field(default=None, alias="type")
    value: Optional[str] = None
    key: Optional[Key] = None
    main: Optional[str] = None
    sevClient: Optional[SevClient] = None
    class Config:
        populate_by_name = True


from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.sevclient import SevClient
from sevdesk.converters.checkaccount import CheckAccount
from sevdesk.converters.sourcetransaction import SourceTransaction
from sevdesk.converters.targettransaction import TargetTransaction

class CheckAccountTransaction(BaseModel):
    id_: Optional[int] = Field(default=None, alias="id")
    objectName: Optional[str] = None
    create: Optional[str] = None
    update: Optional[str] = None
    sevClient: Optional[SevClient] = None
    valueDate: str
    entryDate: Optional[str] = None
    paymtPurpose: Optional[str] = None
    amount: float
    payeePayerName: Optional[str]
    payeePayerAcctNo: Optional[str] = None
    payeePayerBankCode: Optional[str] = None
    checkAccount: CheckAccount
    status: int
    sourceTransaction: Optional[SourceTransaction] = None
    targetTransaction: Optional[TargetTransaction] = None
    class Config:
        populate_by_name = True


from typing import Optional
from pydantic import BaseModel, Field

from sevdesk.converters.sevclient import SevClient

class CreateFileImportAccountResponse(BaseModel):
    id_: Optional[str] = Field(default=None, alias="id")
    objectName: Optional[str] = None
    create: Optional[str] = None
    update: Optional[str] = None
    sevClient: Optional[SevClient] = None
    name: Optional[str] = None
    iban: Optional[str] = None
    type_: Optional[str] = Field(default=None, alias="type")
    importType: Optional[str] = None
    currency: Optional[str] = None
    defaultAccount: Optional[str] = None
    status: Optional[str] = None
    autoMapTransactions: Optional[str] = None
    accountingNumber: Optional[str] = None
    class Config:
        populate_by_name = True